@@ -38,9 +38,7 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-#include <polkit/polkit.h>
-#include <polkit-dbus/polkit-dbus.h>
-#include <packagekit-glib/packagekit.h>
+#include <packagekit-glib2/packagekit.h>
 #include <gconf/gconf-client.h>
 
 #include "egg-debug.h"
@@ -61,6 +59,8 @@ struct GpkDbusPrivate
 	gint			 timeout_tmp;
 	GpkX11			*x11;
 	GPtrArray		*array;
+	DBusGProxy		*proxy_session_pid;
+	DBusGProxy		*proxy_system_pid;
 };
 
 G_DEFINE_TYPE (GpkDbus, gpk_dbus, G_TYPE_OBJECT)
@@ -102,53 +102,125 @@ gpk_dbus_error_get_type (void)
 	return etype;
 }
 
+/**
+ * gpk_dbus_get_pid_session:
+ **/
+static guint
+gpk_dbus_get_pid_session (GpkDbus *dbus, const gchar *sender)
+{
+	guint pid = G_MAXUINT;
+	gboolean ret;
+	GError *error = NULL;
+
+	/* get pid from DBus (quite slow) */
+	ret = dbus_g_proxy_call (dbus->priv->proxy_session_pid, "GetConnectionUnixProcessID", &error,
+				 G_TYPE_STRING, sender,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, &pid,
+				 G_TYPE_INVALID);
+	if (!ret) {
+		egg_debug ("failed to get pid from session: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	return pid;
+}
+
+/**
+ * gpk_dbus_get_pid_system:
+ **/
+static guint
+gpk_dbus_get_pid_system (GpkDbus *dbus, const gchar *sender)
+{
+	guint pid = G_MAXUINT;
+	gboolean ret;
+	GError *error = NULL;
+
+	/* get pid from DBus (quite slow) */
+	ret = dbus_g_proxy_call (dbus->priv->proxy_system_pid, "GetConnectionUnixProcessID", &error,
+				 G_TYPE_STRING, sender,
+				 G_TYPE_INVALID,
+				 G_TYPE_UINT, &pid,
+				 G_TYPE_INVALID);
+	if (!ret) {
+		egg_debug ("failed to get pid from system: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	return pid;
+}
+
+/**
+ * gpk_dbus_get_pid:
+ **/
+static guint
+gpk_dbus_get_pid (GpkDbus *dbus, const gchar *sender)
+{
+	guint pid;
+
+	g_return_val_if_fail (PK_IS_DBUS (dbus), G_MAXUINT);
+	g_return_val_if_fail (dbus->priv->proxy_session_pid != NULL, G_MAXUINT);
+	g_return_val_if_fail (dbus->priv->proxy_system_pid != NULL, G_MAXUINT);
+	g_return_val_if_fail (sender != NULL, G_MAXUINT);
+
+	/* check system bus first */
+	pid = gpk_dbus_get_pid_system (dbus, sender);
+	if (pid != G_MAXUINT)
+		goto out;
+
+	/* and then session bus */
+	pid = gpk_dbus_get_pid_session (dbus, sender);
+	if (pid != G_MAXUINT)
+		goto out;
+
+	/* should be impossible */
+	egg_warning ("could not find pid!");
+out:
+	return pid;
+}
+
+
 /**
  * gpk_dbus_get_exec_for_sender:
  **/
 static gchar *
-gpk_dbus_get_exec_for_sender (const gchar *sender)
+gpk_dbus_get_exec_for_sender (GpkDbus *dbus, const gchar *sender)
 {
-	pid_t pid;
-	gchar exec[128];
-	PolKitCaller *caller = NULL;
-	DBusError dbus_error;
-	gboolean ret = FALSE;
-	gint retval;
-	DBusConnection *connection;
-	gchar *sender_exe = NULL;
+	gboolean ret;
+	gchar *filename = NULL;
+	gchar *cmdline = NULL;
+	GError *error = NULL;
+	guint pid;
 
-	/* get a connection */
-	connection = dbus_bus_get (DBUS_BUS_SESSION, NULL);
-	if (connection == NULL)
-		egg_error ("fatal, no system dbus");
+	g_return_val_if_fail (PK_IS_DBUS (dbus), NULL);
+	g_return_val_if_fail (sender != NULL, NULL);
 
-	dbus_error_init (&dbus_error);
-	caller = polkit_caller_new_from_dbus_name (connection, sender, &dbus_error);
-	if (caller == NULL) {
-		egg_warning ("cannot get caller from sender %s: %s", sender, dbus_error.message);
-		dbus_error_free (&dbus_error);
+	/* get pid */
+	pid = gpk_dbus_get_pid (dbus, sender);
+	if (pid == G_MAXUINT) {
+		egg_warning ("failed to get PID");
 		goto out;
 	}
 
-	ret = polkit_caller_get_pid (caller, &pid);
+	/* get command line from proc */
+	filename = g_strdup_printf ("/proc/%i/cmdline", pid);
+	ret = g_file_get_contents (filename, &cmdline, NULL, &error);
 	if (!ret) {
-		egg_warning ("cannot get pid from sender %p", sender);
-		goto out;
+		egg_warning ("failed to get cmdline: %s", error->message);
+		g_error_free (error);
 	}
 
-	retval = polkit_sysdeps_get_exe_for_pid (pid, exec, 128);
-	if (retval == -1) {
-		egg_warning ("cannot get exec for pid %i", pid);
+	/* if command line contains (deleted) the original binary is invalid */
+	if (g_strstr_len (cmdline, -1, "(deleted)") != NULL) {
+		g_free (cmdline);
+		cmdline = NULL;
 		goto out;
 	}
-
-	/* make a copy */
-	sender_exe = g_strdup (exec);
-
 out:
-	if (caller != NULL)
-		polkit_caller_unref (caller);
-	return sender_exe;
+	g_free (filename);
+	return cmdline;
 }
 
 /**
@@ -165,39 +237,39 @@ gpk_dbus_set_interaction_from_text (PkBitfield *interact, gint *timeout, const g
 
 	/* do special keys first */
 	for (i=0; i<len; i++) {
-		if (egg_strequal (interactions[i], "always"))
+		if (g_strcmp0 (interactions[i], "always") == 0)
 			*interact = GPK_CLIENT_INTERACT_ALWAYS;
-		else if (egg_strequal (interactions[i], "never"))
+		else if (g_strcmp0 (interactions[i], "never") == 0)
 			*interact = GPK_CLIENT_INTERACT_NEVER;
 	}
 
 	/* add or remove from defaults */
 	for (i=0; i<len; i++) {
 		/* show */
-		if (egg_strequal (interactions[i], "show-confirm-search"))
+		if (g_strcmp0 (interactions[i], "show-confirm-search") == 0)
 			pk_bitfield_add (*interact, GPK_CLIENT_INTERACT_CONFIRM_SEARCH);
-		else if (egg_strequal (interactions[i], "show-confirm-deps"))
+		else if (g_strcmp0 (interactions[i], "show-confirm-deps") == 0)
 			pk_bitfield_add (*interact, GPK_CLIENT_INTERACT_CONFIRM_DEPS);
-		else if (egg_strequal (interactions[i], "show-confirm-install"))
+		else if (g_strcmp0 (interactions[i], "show-confirm-install") == 0)
 			pk_bitfield_add (*interact, GPK_CLIENT_INTERACT_CONFIRM_INSTALL);
-		else if (egg_strequal (interactions[i], "show-progress"))
+		else if (g_strcmp0 (interactions[i], "show-progress") == 0)
 			pk_bitfield_add (*interact, GPK_CLIENT_INTERACT_PROGRESS);
-		else if (egg_strequal (interactions[i], "show-finished"))
+		else if (g_strcmp0 (interactions[i], "show-finished") == 0)
 			pk_bitfield_add (*interact, GPK_CLIENT_INTERACT_FINISHED);
-		else if (egg_strequal (interactions[i], "show-warning"))
+		else if (g_strcmp0 (interactions[i], "show-warning") == 0)
 			pk_bitfield_add (*interact, GPK_CLIENT_INTERACT_WARNING);
 		/* hide */
-		else if (egg_strequal (interactions[i], "hide-confirm-search"))
+		else if (g_strcmp0 (interactions[i], "hide-confirm-search") == 0)
 			pk_bitfield_remove (*interact, GPK_CLIENT_INTERACT_CONFIRM_SEARCH);
-		else if (egg_strequal (interactions[i], "hide-confirm-deps"))
+		else if (g_strcmp0 (interactions[i], "hide-confirm-deps") == 0)
 			pk_bitfield_remove (*interact, GPK_CLIENT_INTERACT_CONFIRM_DEPS);
-		else if (egg_strequal (interactions[i], "hide-confirm-install"))
+		else if (g_strcmp0 (interactions[i], "hide-confirm-install") == 0)
 			pk_bitfield_remove (*interact, GPK_CLIENT_INTERACT_CONFIRM_INSTALL);
-		else if (egg_strequal (interactions[i], "hide-progress"))
+		else if (g_strcmp0 (interactions[i], "hide-progress") == 0)
 			pk_bitfield_remove (*interact, GPK_CLIENT_INTERACT_PROGRESS);
-		else if (egg_strequal (interactions[i], "hide-finished"))
+		else if (g_strcmp0 (interactions[i], "hide-finished") == 0)
 			pk_bitfield_remove (*interact, GPK_CLIENT_INTERACT_FINISHED);
-		else if (egg_strequal (interactions[i], "hide-warning"))
+		else if (g_strcmp0 (interactions[i], "hide-warning") == 0)
 			pk_bitfield_remove (*interact, GPK_CLIENT_INTERACT_WARNING);
 		/* wait */
 		else if (g_str_has_prefix (interactions[i], "timeout="))
@@ -264,10 +336,6 @@ gpk_dbus_create_task (GpkDbus *dbus, guint32 xid, const gchar *interaction, DBus
 	egg_debug ("interact=%i", (gint) interact);
 	gpk_dbus_task_set_interaction (task, interact);
 
-	/* set timeout */
-	egg_debug ("timeout=%i", timeout);
-	gpk_dbus_task_set_timeout (task, timeout);
-
 	/* set the parent window */
 	gpk_dbus_task_set_xid (task, xid);
 
@@ -288,8 +356,9 @@ gpk_dbus_create_task (GpkDbus *dbus, guint32 xid, const gchar *interaction, DBus
 
 	/* get the program name and set */
 	sender = dbus_g_method_get_sender (context);
-	exec = gpk_dbus_get_exec_for_sender (sender);
-	gpk_dbus_task_set_exec (task, exec);
+	exec = gpk_dbus_get_exec_for_sender (dbus, sender);
+	if (exec != NULL)
+		gpk_dbus_task_set_exec (task, exec);
 
 	/* unref on delete */
 	//g_signal_connect...
@@ -347,6 +416,17 @@ gpk_dbus_install_provide_files (GpkDbus *dbus, guint32 xid, gchar **files, const
 	gpk_dbus_task_install_provide_files (task, files);
 }
 
+/**
+ * gpk_dbus_remove_package_by_file:
+ **/
+void
+gpk_dbus_remove_package_by_file (GpkDbus *dbus, guint32 xid, gchar **files, const gchar *interaction, DBusGMethodInvocation *context)
+{
+	GpkDbusTask *task;
+	task = gpk_dbus_create_task (dbus, xid, interaction, context);
+	gpk_dbus_task_remove_package_by_file (task, files);
+}
+
 /**
  * gpk_dbus_install_catalogs:
  **/
@@ -421,11 +501,26 @@ gpk_dbus_class_init (GpkDbusClass *klass)
 static void
 gpk_dbus_init (GpkDbus *dbus)
 {
+	DBusGConnection *connection;
+
 	dbus->priv = GPK_DBUS_GET_PRIVATE (dbus);
 	dbus->priv->timeout_tmp = -1;
 	dbus->priv->gconf_client = gconf_client_get_default ();
-	dbus->priv->array = g_ptr_array_new ();
+	dbus->priv->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	dbus->priv->x11 = gpk_x11_new ();
+
+	/* find out PIDs on the session bus */
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+	dbus->priv->proxy_session_pid = dbus_g_proxy_new_for_name_owner (connection,
+								 "org.freedesktop.DBus",
+								 "/org/freedesktop/DBus/Bus",
+								 "org.freedesktop.DBus", NULL);
+	/* find out PIDs on the system bus */
+	connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, NULL);
+	dbus->priv->proxy_system_pid = dbus_g_proxy_new_for_name_owner (connection,
+								 "org.freedesktop.DBus",
+								 "/org/freedesktop/DBus/Bus",
+								 "org.freedesktop.DBus", NULL);
 }
 
 /**
@@ -440,10 +535,11 @@ gpk_dbus_finalize (GObject *object)
 
 	dbus = GPK_DBUS (object);
 	g_return_if_fail (dbus->priv != NULL);
-	g_ptr_array_foreach (dbus->priv->array, (GFunc) g_object_unref, NULL);
-	g_ptr_array_free (dbus->priv->array, TRUE);
+	g_ptr_array_unref (dbus->priv->array);
 	g_object_unref (dbus->priv->gconf_client);
 	g_object_unref (dbus->priv->x11);
+	g_object_unref (dbus->priv->proxy_session_pid);
+	g_object_unref (dbus->priv->proxy_system_pid);
 
 	G_OBJECT_CLASS (gpk_dbus_parent_class)->finalize (object);
 }
