@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -24,7 +24,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <packagekit-glib/packagekit.h>
+#include <packagekit-glib2/packagekit.h>
 #include <locale.h>
 #include <string.h>
 
@@ -90,10 +90,6 @@ gpk_desktop_get_menu_path_submenu (const gchar *filename, const gchar *menu_file
 		goto out;
 	}
 	directory = gmenu_tree_get_root_directory (tree);
-	if (directory == NULL) {
-		egg_warning ("no directory for %s", menu_file);
-		goto out;
-	}
 	text = gpk_desktop_directory_get_menu_path (directory, filename);
 	if (text == NULL) {
 		egg_debug ("no path for %s", filename);
@@ -271,10 +267,8 @@ gpk_desktop_guess_best_file (PkDesktop *desktop, const gchar *package)
 	best_file = g_strdup (g_ptr_array_index (array, max_index));
 	egg_debug ("using %s", best_file);
 out:
-	if (array != NULL) {
-		g_ptr_array_foreach (array, (GFunc) g_free, NULL);
-		g_ptr_array_free (array, TRUE);
-	}
+	if (array != NULL)
+		g_ptr_array_unref (array);
 	return best_file;
 }
 
