@@ -25,30 +25,25 @@
 #include <glib/gi18n.h>
 
 #include <gtk/gtk.h>
-#include <math.h>
-#include <string.h>
-#include <dbus/dbus-glib.h>
 #include <locale.h>
 #include <sys/types.h>
 #include <pwd.h>
 
 #include <gconf/gconf-client.h>
-#include <packagekit-glib/packagekit.h>
+#include <packagekit-glib2/packagekit.h>
 #include <unique/unique.h>
 
 #include "egg-debug.h"
-#include "egg-string.h"
 
 #include "gpk-common.h"
 #include "gpk-gnome.h"
-#include "gpk-enum.h"
 
 static GtkBuilder *builder = NULL;
 static GtkListStore *list_store = NULL;
 static PkClient *client = NULL;
 static gchar *transaction_id = NULL;
 static gchar *filter = NULL;
-static PkObjList *transactions = NULL;
+static GPtrArray *transactions = NULL;
 static GtkTreePath *path_global = NULL;
 
 enum
@@ -193,7 +188,6 @@ gpk_log_get_type_line (gchar **array, PkInfoEnum info)
 	gchar *text;
 	gchar *whole;
 	gchar **sections;
-	PkPackageId *id;
 
 	string = g_string_new ("");
 	size = g_strv_length (array);
@@ -204,11 +198,9 @@ gpk_log_get_type_line (gchar **array, PkInfoEnum info)
 		sections = g_strsplit (array[i], "\t", 0);
 		info_local = pk_info_enum_from_text (sections[0]);
 		if (info_local == info) {
-			id = pk_package_id_new_from_string (sections[1]);
-			text = gpk_package_id_format_oneline (id, NULL);
+			text = gpk_package_id_format_oneline (sections[1], NULL);
 			g_string_append_printf (string, "%s, ", text);
 			g_free (text);
-			pk_package_id_free (id);
 		}
 		g_strfreev (sections);
 	}
@@ -387,18 +379,30 @@ gpk_log_message_received_cb (UniqueApp *app, UniqueCommand command, UniqueMessag
  * gpk_log_filter:
  **/
 static gboolean
-gpk_log_filter (const PkTransactionObj *obj)
+gpk_log_filter (PkTransactionPast *item)
 {
 	gboolean ret = FALSE;
 	guint i;
 	guint length;
 	gchar **sections;
 	gchar **packages;
-	PkPackageId *id;
+	gchar **split;
+	gchar *tid;
+	gboolean succeeded;
+	gchar *cmdline;
+	gchar *data;
+
+	/* get data */
+	g_object_get (item,
+		      "tid", &tid,
+		      "succeeded", &succeeded,
+		      "cmdline", &cmdline,
+		      "data", &data,
+		      NULL);
 
 	/* only show transactions that succeeded */
-	if (!obj->succeeded) {
-		egg_debug ("tid %s did not succeed, so not adding", obj->tid);
+	if (!succeeded) {
+		egg_debug ("tid %s did not succeed, so not adding", tid);
 		return FALSE;
 	}
 
@@ -406,11 +410,11 @@ gpk_log_filter (const PkTransactionObj *obj)
 		return TRUE;
 
 	/* matches cmdline */
-	if (obj->cmdline != NULL && g_strrstr (obj->cmdline, filter) != NULL)
+	if (cmdline != NULL && g_strrstr (cmdline, filter) != NULL)
 		ret = TRUE;
 
 	/* look in all the data for the filter string */
-	packages = g_strsplit (obj->data, "\n", 0);
+	packages = g_strsplit (data, "\n", 0);
 	length = g_strv_length (packages);
 	for (i=0; i<length; i++) {
 		sections = g_strsplit (packages[i], "\t", 0);
@@ -420,15 +424,15 @@ gpk_log_filter (const PkTransactionObj *obj)
 			ret = TRUE;
 
 		/* check to see if package name, version or arch matches */
-		id = pk_package_id_new_from_string (sections[1]);
-		if (g_strrstr (id->name, filter) != NULL)
+		split = pk_package_id_split (sections[1]);
+		if (g_strrstr (split[0], filter) != NULL)
 			ret = TRUE;
-		if (id->version != NULL && g_strrstr (id->version, filter) != NULL)
+		if (split[1] != NULL && g_strrstr (split[1], filter) != NULL)
 			ret = TRUE;
-		if (id->arch != NULL && g_strrstr (id->arch, filter) != NULL)
+		if (split[2] != NULL && g_strrstr (split[2], filter) != NULL)
 			ret = TRUE;
 
-		pk_package_id_free (id);
+		g_strfreev (split);
 		g_strfreev (sections);
 
 		/* shortcut for speed */
@@ -436,16 +440,19 @@ gpk_log_filter (const PkTransactionObj *obj)
 			break;
 	}
 
+	g_free (tid);
+	g_free (cmdline);
+	g_free (data);
 	g_strfreev (packages);
 
 	return ret;
 }
 
 /**
- * gpk_log_add_obj
+ * gpk_log_add_item
  **/
 static void
-gpk_log_add_obj (const PkTransactionObj *obj)
+gpk_log_add_item (PkTransactionPast *item)
 {
 	GtkTreeIter iter;
 	gchar *details;
@@ -457,19 +464,39 @@ gpk_log_add_obj (const PkTransactionObj *obj)
 	const gchar *tool;
 	static guint count;
 	struct passwd *pw;
+	gchar *tid;
+	gchar *timespec;
+	gboolean succeeded;
+	guint duration;
+	gchar *cmdline;
+	guint uid;
+	gchar *data;
+	PkRoleEnum role;
 	GtkTreeView *treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview_simple"));
 	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
 
+	/* get data */
+	g_object_get (item,
+		      "role", &role,
+		      "tid", &tid,
+		      "timespec", &timespec,
+		      "succeeded", &succeeded,
+		      "duration", &duration,
+		      "cmdline", &cmdline,
+		      "uid", &uid,
+		      "data", &data,
+		      NULL);
+
 	/* put formatted text into treeview */
-	details = gpk_log_get_details_localised (obj->timespec, obj->data);
-	date = gpk_log_get_localised_date (obj->timespec);
+	details = gpk_log_get_details_localised (timespec, data);
+	date = gpk_log_get_localised_date (timespec);
 	date_part = g_strsplit (date, ", ", 2);
 
-	icon_name = gpk_role_enum_to_icon_name (obj->role);
-	role_text = gpk_role_enum_to_localised_past (obj->role);
+	icon_name = gpk_role_enum_to_icon_name (role);
+	role_text = gpk_role_enum_to_localised_past (role);
 
 	/* query real name */
-	pw = getpwuid(obj->uid);
+	pw = getpwuid(uid);
 	if (pw != NULL) {
 		if (pw->pw_gecos != NULL)
 			username = pw->pw_gecos;
@@ -478,29 +505,29 @@ gpk_log_add_obj (const PkTransactionObj *obj)
 	}
 
 	/* get nice name for tool name */
-	if (g_strcmp0 (obj->cmdline, "pkcon") == 0)
+	if (g_strcmp0 (cmdline, "pkcon") == 0)
 		/* TRANSLATORS: short name for pkcon */
 		tool = _("Command line client");
-	else if (g_strcmp0 (obj->cmdline, "gpk-application") == 0)
+	else if (g_strcmp0 (cmdline, "gpk-application") == 0)
 		/* TRANSLATORS: short name for gpk-update-viewer */
 		tool = _("Add/Remove Software");
-	else if (g_strcmp0 (obj->cmdline, "gpk-update-viewer") == 0)
+	else if (g_strcmp0 (cmdline, "gpk-update-viewer") == 0)
 		/* TRANSLATORS: short name for gpk-update-viewer */
 		tool = _("Update System");
-	else if (g_strcmp0 (obj->cmdline, "gpk-update-icon") == 0)
+	else if (g_strcmp0 (cmdline, "gpk-update-icon") == 0)
 		/* TRANSLATORS: short name for gpk-update-icon */
 		tool = _("Update Icon");
 	else
-		tool = obj->cmdline;
+		tool = cmdline;
 
-	gpk_log_model_get_iter (model, &iter, obj->tid);
+	gpk_log_model_get_iter (model, &iter, tid);
 	gtk_list_store_set (list_store, &iter,
 			    GPK_LOG_COLUMN_ICON, icon_name,
-			    GPK_LOG_COLUMN_TIMESPEC, obj->timespec,
+			    GPK_LOG_COLUMN_TIMESPEC, timespec,
 			    GPK_LOG_COLUMN_DATE, date_part[1],
 			    GPK_LOG_COLUMN_ROLE, role_text,
 			    GPK_LOG_COLUMN_DETAILS, details,
-			    GPK_LOG_COLUMN_ID, obj->tid,
+			    GPK_LOG_COLUMN_ID, tid,
 			    GPK_LOG_COLUMN_USER, username,
 			    GPK_LOG_COLUMN_TOOL, tool,
 			    GPK_LOG_COLUMN_ACTIVE, TRUE, -1);
@@ -511,6 +538,10 @@ gpk_log_add_obj (const PkTransactionObj *obj)
 			gtk_main_iteration ();
 
 	g_strfreev (date_part);
+	g_free (tid);
+	g_free (timespec);
+	g_free (cmdline);
+	g_free (data);
 	g_free (details);
 	g_free (date);
 }
@@ -523,7 +554,7 @@ gpk_log_refilter (void)
 {
 	guint i;
 	gboolean ret;
-	const PkTransactionObj *obj;
+	PkTransactionPast *item;
 	GtkWidget *widget;
 	const gchar *package;
 	GtkTreeView *treeview;
@@ -533,7 +564,7 @@ gpk_log_refilter (void)
 	g_free (filter);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "entry_package"));
 	package = gtk_entry_get_text (GTK_ENTRY(widget));
-	if (!egg_strzero (package))
+	if (package != NULL && package[0] != '\0')
 		filter = g_strdup (package);
 	else
 		filter = NULL;
@@ -547,10 +578,10 @@ gpk_log_refilter (void)
 
 	/* go through the list, adding and removing the items as required */
 	for (i=0; i<transactions->len; i++) {
-		obj = pk_obj_list_index (transactions, i);
-		ret = gpk_log_filter (obj);
+		item = g_ptr_array_index (transactions, i);
+		ret = gpk_log_filter (item);
 		if (ret)
-			gpk_log_add_obj (obj);
+			gpk_log_add_item (item);
 	}
 
 	/* remove the items that are not used */
@@ -558,38 +589,52 @@ gpk_log_refilter (void)
 }
 
 /**
- * gpk_log_refresh
+ * gpk_log_get_old_transactions_cb
  **/
-static gboolean
-gpk_log_refresh (void)
+static void
+gpk_log_get_old_transactions_cb (GObject *object, GAsyncResult *res, gpointer user_data)
 {
-	gboolean ret;
+//	PkClient *client = PK_CLIENT (object);
 	GError *error = NULL;
+	PkResults *results = NULL;
+	PkError *error_code = NULL;
 
-	ret = pk_client_reset (client, &error);
-	if (!ret) {
-		egg_warning ("failed to reset client: %s", error->message);
+	/* get the results */
+	results = pk_client_generic_finish (client, res, &error);
+	if (results == NULL) {
+		egg_warning ("failed to get old transactions: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-	ret = pk_client_get_old_transactions (client, 0, &error);
-	if (!ret) {
-		egg_warning ("failed to get list: %s", error->message);
-		g_error_free (error);
+
+	/* check error code */
+	error_code = pk_results_get_error_code (results);
+	if (error_code != NULL) {
+		egg_warning ("failed to get old transactions: %s, %s", pk_error_enum_to_text (pk_error_get_code (error_code)), pk_error_get_details (error_code));
 		goto out;
 	}
 
 	/* get the list */
 	if (transactions != NULL)
-		g_object_unref (transactions);
-#if PK_CHECK_VERSION(0,5,0)
-	transactions = pk_client_get_transaction_list (client);
-#else
-	transactions = pk_client_get_cached_objects (client); /* removed in 0.5.x */
-#endif
+		g_ptr_array_unref (transactions);
+	transactions = pk_results_get_transaction_array (results);
 	gpk_log_refilter ();
 out:
-	return ret;
+	if (error_code != NULL)
+		g_object_unref (error_code);
+	if (results != NULL)
+		g_object_unref (results);
+}
+
+/**
+ * gpk_log_refresh
+ **/
+static void
+gpk_log_refresh (void)
+{
+	/* get the list async */
+	pk_client_get_old_transactions_async (client, 0, NULL, NULL, NULL,
+					      (GAsyncReadyCallback) gpk_log_get_old_transactions_cb, NULL);
 }
 
 /**
@@ -627,7 +672,6 @@ gpk_log_entry_filter_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_da
 int
 main (int argc, char *argv[])
 {
-	gboolean verbose = FALSE;
 	GOptionContext *context;
 	GConfClient *gconf_client;
 	GtkWidget *widget;
@@ -640,8 +684,6 @@ main (int argc, char *argv[])
 	GError *error = NULL;
 
 	const GOptionEntry options[] = {
-		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
-		  N_("Show extra debugging information"), NULL },
 		{ "filter", 'f', 0, G_OPTION_ARG_STRING, &filter,
 		  /* TRANSLATORS: preset the GtktextBox with this filter text */
 		  N_("Set the filter to this value"), NULL },
@@ -659,18 +701,17 @@ main (int argc, char *argv[])
 
 	if (! g_thread_supported ())
 		g_thread_init (NULL);
-	dbus_g_thread_init ();
 	g_type_init ();
+	gtk_init (&argc, &argv);
 
 	context = g_option_context_new (NULL);
 	g_option_context_set_summary (context, _("Software Log Viewer"));
 	g_option_context_add_main_entries (context, options, NULL);
+	g_option_context_add_group (context, egg_debug_get_option_group ());
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, NULL);
 	g_option_context_free (context);
 
-	egg_debug_init (verbose);
-	gtk_init (&argc, &argv);
-
 	/* are we running privileged */
 	ret = gpk_check_privileged_user (_("Log viewer"), TRUE);
 	if (!ret)
@@ -691,13 +732,14 @@ main (int argc, char *argv[])
 					   GPK_DATA G_DIR_SEPARATOR_S "icons");
 
 	client = pk_client_new ();
-	pk_client_set_use_buffer (client, TRUE, NULL);
-	pk_client_set_synchronous (client, TRUE, NULL);
+	g_object_set (client,
+		      "background", FALSE,
+		      NULL);
 
 	/* get UI */
 	builder = gtk_builder_new ();
 	retval = gtk_builder_add_from_file (builder, GPK_DATA "/gpk-log.ui", &error);
-	if (error != NULL) {
+	if (retval == 0) {
 		egg_warning ("failed to load ui: %s", error->message);
 		g_error_free (error);
 		goto out_build;
@@ -726,6 +768,7 @@ main (int argc, char *argv[])
 	g_signal_connect (widget, "clicked", G_CALLBACK (gpk_log_button_help_cb), NULL);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_refresh"));
 	g_signal_connect (widget, "clicked", G_CALLBACK (gpk_log_button_refresh_cb), NULL);
+	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_filter"));
 	g_signal_connect (widget, "clicked", G_CALLBACK (gpk_log_button_filter_cb), NULL);
 
@@ -768,9 +811,6 @@ main (int argc, char *argv[])
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list_store),
 					      GPK_LOG_COLUMN_TIMESPEC, GTK_SORT_DESCENDING);
 
-	/* get the update list */
-	gpk_log_refresh ();
-
 	/* show */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_simple"));
 	gtk_widget_show (widget);
@@ -781,6 +821,9 @@ main (int argc, char *argv[])
 		gpk_window_set_parent_xid (GTK_WINDOW (widget), xid);
 	}
 
+	/* get the update list */
+	gpk_log_refresh ();
+
 	gtk_main ();
 
 out_build:
@@ -790,7 +833,7 @@ out_build:
 	g_free (transaction_id);
 	g_free (filter);
 	if (transactions != NULL)
-		g_object_unref (transactions);
+		g_ptr_array_unref (transactions);
 unique_out:
 	g_object_unref (unique_app);
 	return 0;
