@@ -31,7 +31,7 @@
 #include <string.h>
 #include <dbus/dbus-glib.h>
 #include <gconf/gconf-client.h>
-#include <packagekit-glib/packagekit.h>
+#include <packagekit-glib2/packagekit.h>
 #include <unique/unique.h>
 
 #include <gpk-common.h>
@@ -56,13 +56,18 @@
 /* TRANSLATORS: don't update anything */
 #define PK_UPDATE_NONE_TEXT		_("Nothing")
 
+#define GPK_PREFS_VALUE_NEVER		(0)
+#define GPK_PREFS_VALUE_HOURLY		(60*60)
+#define GPK_PREFS_VALUE_DAILY		(60*60*24)
+#define GPK_PREFS_VALUE_WEEKLY		(60*60*24*7)
+
 static GtkBuilder *builder = NULL;
 
 /**
- * pk_button_help_cb:
+ * gpk_prefs_help_cb:
  **/
 static void
-pk_button_help_cb (GtkWidget *widget, gpointer data)
+gpk_prefs_help_cb (GtkWidget *widget, gpointer data)
 {
 	gpk_gnome_help ("prefs");
 }
@@ -94,26 +99,24 @@ static void
 gpk_prefs_update_freq_combo_changed (GtkWidget *widget, gpointer data)
 {
 	gchar *value;
-	const gchar *action;
-	GpkFreqEnum freq = GPK_FREQ_ENUM_UNKNOWN;
+	guint freq = 0;
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 	if (strcmp (value, PK_FREQ_HOURLY_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_HOURLY;
+		freq = GPK_PREFS_VALUE_HOURLY;
 	else if (strcmp (value, PK_FREQ_DAILY_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_DAILY;
+		freq = GPK_PREFS_VALUE_DAILY;
 	else if (strcmp (value, PK_FREQ_WEEKLY_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_WEEKLY;
+		freq = GPK_PREFS_VALUE_WEEKLY;
 	else if (strcmp (value, PK_FREQ_NEVER_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_NEVER;
+		freq = GPK_PREFS_VALUE_NEVER;
 	else
 		g_assert (FALSE);
 
-	action = gpk_freq_enum_to_text (freq);
-	egg_debug ("Changing %s to %s", GPK_CONF_FREQUENCY_GET_UPDATES, action);
-	gconf_client_set_string (client, GPK_CONF_FREQUENCY_GET_UPDATES, action, NULL);
+	egg_debug ("Changing %s to %i", GPK_CONF_FREQUENCY_GET_UPDATES, freq);
+	gconf_client_set_int (client, GPK_CONF_FREQUENCY_GET_UPDATES, freq, NULL);
 	g_free (value);
 	g_object_unref (client);
 }
@@ -125,24 +128,22 @@ static void
 gpk_prefs_upgrade_freq_combo_changed (GtkWidget *widget, gpointer data)
 {
 	gchar *value;
-	const gchar *action;
-	GpkFreqEnum freq = GPK_FREQ_ENUM_UNKNOWN;
+	guint freq = 0;
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 	if (strcmp (value, PK_FREQ_DAILY_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_DAILY;
+		freq = GPK_PREFS_VALUE_DAILY;
 	else if (strcmp (value, PK_FREQ_WEEKLY_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_WEEKLY;
+		freq = GPK_PREFS_VALUE_WEEKLY;
 	else if (strcmp (value, PK_FREQ_NEVER_TEXT) == 0)
-		freq = GPK_FREQ_ENUM_NEVER;
+		freq = GPK_PREFS_VALUE_NEVER;
 	else
 		g_assert (FALSE);
 
-	action = gpk_freq_enum_to_text (freq);
-	egg_debug ("Changing %s to %s", GPK_CONF_FREQUENCY_GET_UPGRADES, action);
-	gconf_client_set_string (client, GPK_CONF_FREQUENCY_GET_UPGRADES, action, NULL);
+	egg_debug ("Changing %s to %i", GPK_CONF_FREQUENCY_GET_UPGRADES, freq);
+	gconf_client_set_int (client, GPK_CONF_FREQUENCY_GET_UPGRADES, freq, NULL);
 	g_free (value);
 	g_object_unref (client);
 }
@@ -157,7 +158,6 @@ gpk_prefs_update_combo_changed (GtkWidget *widget, gpointer data)
 	const gchar *action;
 	GpkUpdateEnum update = GPK_UPDATE_ENUM_UNKNOWN;
 	GConfClient *client;
-	GtkWidget *notify_widget;
 
 	client = gconf_client_get_default ();
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
@@ -165,16 +165,12 @@ gpk_prefs_update_combo_changed (GtkWidget *widget, gpointer data)
 		egg_warning ("value NULL");
 		return;
 	}
-	notify_widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_notify_updates"));
 	if (strcmp (value, PK_UPDATE_ALL_TEXT) == 0) {
 		update = GPK_UPDATE_ENUM_ALL;
-		gtk_widget_set_sensitive (notify_widget, FALSE);
 	} else if (strcmp (value, PK_UPDATE_SECURITY_TEXT) == 0) {
 		update = GPK_UPDATE_ENUM_SECURITY;
-		gtk_widget_set_sensitive (notify_widget, TRUE);
 	} else if (strcmp (value, PK_UPDATE_NONE_TEXT) == 0) {
 		update = GPK_UPDATE_ENUM_NONE;
-		gtk_widget_set_sensitive (notify_widget, TRUE);
 	} else {
 		g_assert (FALSE);
 	}
@@ -212,23 +208,16 @@ gpk_prefs_update_freq_combo_simple_text (GtkWidget *combo_box)
 static void
 gpk_prefs_update_freq_combo_setup (void)
 {
-	gchar *value;
+	guint value;
 	gboolean is_writable;
 	GtkWidget *widget;
-	GpkFreqEnum freq;
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_check"));
 	is_writable = gconf_client_key_is_writable (client, GPK_CONF_FREQUENCY_GET_UPDATES, NULL);
-	value = gconf_client_get_string (client, GPK_CONF_FREQUENCY_GET_UPDATES, NULL);
-	if (value == NULL) {
-		egg_warning ("invalid schema, please re-install");
-		return;
-	}
-	egg_debug ("value from gconf %s", value);
-	freq = gpk_freq_enum_from_text (value);
-	g_free (value);
+	value = gconf_client_get_int (client, GPK_CONF_FREQUENCY_GET_UPDATES, NULL);
+	egg_debug ("value from gconf %i", value);
 	g_object_unref (client);
 
 	/* do we have permission to write? */
@@ -240,8 +229,16 @@ gpk_prefs_update_freq_combo_setup (void)
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), PK_FREQ_DAILY_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), PK_FREQ_WEEKLY_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), PK_FREQ_NEVER_TEXT);
-	/* we can do this as it's the same order */
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), freq);
+
+	/* select the correct entry */
+	if (value == GPK_PREFS_VALUE_HOURLY)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+	else if (value == GPK_PREFS_VALUE_DAILY)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
+	else if (value == GPK_PREFS_VALUE_WEEKLY)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 2);
+	else if (value == GPK_PREFS_VALUE_NEVER)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 3);
 
 	/* only do this after else we redraw the window */
 	g_signal_connect (G_OBJECT (widget), "changed",
@@ -254,23 +251,16 @@ gpk_prefs_update_freq_combo_setup (void)
 static void
 gpk_prefs_upgrade_freq_combo_setup (void)
 {
-	gchar *value;
+	guint value;
 	gboolean is_writable;
 	GtkWidget *widget;
-	GpkFreqEnum freq;
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_upgrade"));
 	is_writable = gconf_client_key_is_writable (client, GPK_CONF_FREQUENCY_GET_UPGRADES, NULL);
-	value = gconf_client_get_string (client, GPK_CONF_FREQUENCY_GET_UPGRADES, NULL);
-	if (value == NULL) {
-		egg_warning ("invalid schema, please re-install");
-		return;
-	}
-	egg_debug ("value from gconf %s", value);
-	freq = gpk_freq_enum_from_text (value);
-	g_free (value);
+	value = gconf_client_get_int (client, GPK_CONF_FREQUENCY_GET_UPGRADES, NULL);
+	egg_debug ("value from gconf %i", value);
 	g_object_unref (client);
 
 	/* do we have permission to write? */
@@ -281,8 +271,14 @@ gpk_prefs_upgrade_freq_combo_setup (void)
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), PK_FREQ_DAILY_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), PK_FREQ_WEEKLY_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), PK_FREQ_NEVER_TEXT);
-	/* don't do daily */
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), freq - 1);
+
+	/* select the correct entry */
+	if (value == GPK_PREFS_VALUE_DAILY)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+	else if (value == GPK_PREFS_VALUE_WEEKLY)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
+	else if (value == GPK_PREFS_VALUE_NEVER)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 2);
 
 	/* only do this after else we redraw the window */
 	g_signal_connect (G_OBJECT (widget), "changed",
@@ -363,14 +359,18 @@ gpk_prefs_message_received_cb (UniqueApp *app, UniqueCommand command, UniqueMess
 }
 
 /**
- * gpk_prefs_network_status_changed_cb:
+ * gpk_prefs_notify_network_state_cb:
  **/
 static void
-gpk_prefs_network_status_changed_cb (PkControl *control, PkNetworkEnum state, gpointer data)
+gpk_prefs_notify_network_state_cb (PkControl *control, GParamSpec *pspec, gpointer data)
 {
 	GtkWidget *widget;
+	PkNetworkEnum state;
 
 	/* only show label on mobile broadband */
+	g_object_get (control,
+		      "network-state", &state,
+		      NULL);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox_mobile_broadband"));
 	if (state == PK_NETWORK_ENUM_MOBILE)
 		gtk_widget_show (widget);
@@ -378,29 +378,115 @@ gpk_prefs_network_status_changed_cb (PkControl *control, PkNetworkEnum state, gp
 		gtk_widget_hide (widget);
 }
 
+/**
+ * pk_prefs_get_properties_cb:
+ **/
+static void
+pk_prefs_get_properties_cb (GObject *object, GAsyncResult *res, GMainLoop *loop)
+{
+	GtkWidget *widget;
+	GError *error = NULL;
+	PkControl *control = PK_CONTROL(object);
+	gboolean ret;
+	PkBitfield roles;
+
+	/* get the result */
+	ret = pk_control_get_properties_finish (control, res, &error);
+	if (!ret) {
+		/* TRANSLATORS: backend is broken, and won't tell us what it supports */
+		g_print ("%s: %s\n", _("Exiting as backend details could not be retrieved"), error->message);
+		g_error_free (error);
+		g_main_loop_quit (loop);
+		goto out;
+	}
+
+	/* get values */
+	g_object_get (control,
+		      "roles", &roles,
+		      NULL);
+
+	/* hide if not supported */
+	if (!pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_DISTRO_UPGRADES)) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_upgrade"));
+		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_upgrade"));
+		gtk_widget_hide (widget);
+	}
+out:
+	return;
+}
+
+/**
+ * pk_prefs_get_network_state_cb:
+ **/
+static void
+pk_prefs_get_network_state_cb (GObject *object, GAsyncResult *res, GMainLoop *loop)
+{
+	GtkWidget *widget;
+	GError *error = NULL;
+	PkControl *control = PK_CONTROL(object);
+	PkNetworkEnum state;
+
+	/* get the result */
+	state = pk_control_get_network_state_finish (control, res, &error);
+	if (state == PK_NETWORK_ENUM_UNKNOWN) {
+		egg_warning ("network status unknown: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* only show label on mobile broadband */
+	if (state == PK_NETWORK_ENUM_MOBILE) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox_mobile_broadband"));
+		gtk_widget_show (widget);
+	}
+out:
+	return;
+}
+
+/**
+ * gpk_prefs_close_cb:
+ **/
+static void
+gpk_prefs_close_cb (GtkWidget *widget, gpointer data)
+{
+	GMainLoop *loop = (GMainLoop *) data;
+	g_main_loop_quit (loop);
+}
+
+/**
+ * gpk_prefs_delete_event_cb:
+ **/
+static gboolean
+gpk_prefs_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+	gpk_prefs_close_cb (widget, data);
+	return FALSE;
+}
+
 /**
  * main:
  **/
 int
 main (int argc, char *argv[])
 {
-	gboolean verbose = FALSE;
 	gboolean program_version = FALSE;
 	GOptionContext *context;
 	GtkWidget *main_window;
 	GtkWidget *widget;
-	PkBitfield roles;
 	PkControl *control;
 	UniqueApp *unique_app;
-	PkNetworkEnum state;
 	guint retval;
+	guint xid = 0;
 	GError *error = NULL;
+	GMainLoop *loop;
 
 	const GOptionEntry options[] = {
-		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
-		  _("Show extra debugging information"), NULL },
 		{ "version", '\0', 0, G_OPTION_ARG_NONE, &program_version,
 		  _("Show the program version and exit"), NULL },
+		{ "parent-window", 'p', 0, G_OPTION_ARG_INT, &xid,
+		  /* TRANSLATORS: we can make this modal (stay on top of) another window */
+		  _("Set the parent window to make this modal"), NULL },
 		{ NULL}
 	};
 
@@ -414,11 +500,14 @@ main (int argc, char *argv[])
 		g_thread_init (NULL);
 	dbus_g_thread_init ();
 	g_type_init ();
+	gtk_init (&argc, &argv);
 
 	context = g_option_context_new (NULL);
 	/* TRANSLATORS: program name, an application to set per-user policy for updates */
 	g_option_context_set_summary(context, _("Software Update Preferences"));
 	g_option_context_add_main_entries (context, options, NULL);
+	g_option_context_add_group (context, egg_debug_get_option_group ());
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, NULL);
 	g_option_context_free (context);
 
@@ -427,9 +516,6 @@ main (int argc, char *argv[])
 		return 0;
 	}
 
-	egg_debug_init (verbose);
-	gtk_init (&argc, &argv);
-
 	/* are we already activated? */
 	unique_app = unique_app_new ("org.freedesktop.PackageKit.Prefs", NULL);
 	if (unique_app_is_running (unique_app)) {
@@ -441,16 +527,15 @@ main (int argc, char *argv[])
 			  G_CALLBACK (gpk_prefs_message_received_cb), NULL);
 
 	/* get actions */
+	loop = g_main_loop_new (NULL, FALSE);
 	control = pk_control_new ();
-	g_signal_connect (control, "network-state-changed",
-			  G_CALLBACK (gpk_prefs_network_status_changed_cb), NULL);
-	roles = pk_control_get_actions (control, NULL);
-	state = pk_control_get_network_state (control, NULL);
+	g_signal_connect (control, "notify::network-state",
+			  G_CALLBACK (gpk_prefs_notify_network_state_cb), NULL);
 
 	/* get UI */
 	builder = gtk_builder_new ();
 	retval = gtk_builder_add_from_file (builder, GPK_DATA "/gpk-prefs.ui", &error);
-	if (error != NULL) {
+	if (retval == 0) {
 		egg_warning ("failed to load ui: %s", error->message);
 		g_error_free (error);
 		goto out_build;
@@ -461,50 +546,41 @@ main (int argc, char *argv[])
 	/* Hide window first so that the dialogue resizes itself without redrawing */
 	gtk_widget_hide (main_window);
 	gtk_window_set_icon_name (GTK_WINDOW (main_window), GPK_ICON_SOFTWARE_UPDATE_PREFS);
-
-	/* Get the main window quit */
-	g_signal_connect_swapped (main_window, "delete_event", G_CALLBACK (gtk_main_quit), NULL);
-
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_notify_updates"));
-	gpk_prefs_notify_checkbutton_setup (widget, GPK_CONF_NOTIFY_AVAILABLE);
-
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_notify_completed"));
-	gpk_prefs_notify_checkbutton_setup (widget, GPK_CONF_NOTIFY_COMPLETED);
+	g_signal_connect (main_window, "delete_event",
+			  G_CALLBACK (gpk_prefs_delete_event_cb), loop);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_mobile_broadband"));
 	gpk_prefs_notify_checkbutton_setup (widget, GPK_CONF_CONNECTION_USE_MOBILE);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_close"));
-	g_signal_connect_swapped (widget, "clicked", G_CALLBACK (gtk_main_quit), NULL);
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gpk_prefs_close_cb), loop);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_help"));
 	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (pk_button_help_cb), NULL);
-
-	/* only show label on mobile broadband */
-	if (state == PK_NETWORK_ENUM_MOBILE) {
-		widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox_mobile_broadband"));
-		gtk_widget_show (widget);
-	}
+			  G_CALLBACK (gpk_prefs_help_cb), NULL);
 
 	/* update the combo boxes */
 	gpk_prefs_update_freq_combo_setup ();
 	gpk_prefs_upgrade_freq_combo_setup ();
 	gpk_prefs_auto_update_combo_setup ();
 
-	/* hide if not supported */
-	if (!pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_DISTRO_UPGRADES)) {
-		widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_upgrade"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_upgrade"));
-		gtk_widget_hide (widget);
-	}
-
 	gtk_widget_show (main_window);
 
+	/* set the parent window if it is specified */
+	if (xid != 0) {
+		egg_debug ("Setting xid %i", xid);
+		gpk_window_set_parent_xid (GTK_WINDOW (main_window), xid);
+	}
+
+	/* get some data */
+	pk_control_get_properties_async (control, NULL, (GAsyncReadyCallback) pk_prefs_get_properties_cb, loop);
+	pk_control_get_network_state_async (control, NULL, (GAsyncReadyCallback) pk_prefs_get_network_state_cb, loop);
+
 	/* wait */
-	gtk_main ();
+	g_main_loop_run (loop);
 
 out_build:
+	g_main_loop_unref (loop);
 	g_object_unref (control);
 	g_object_unref (builder);
 unique_out:
