@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2007-2008 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -21,25 +21,44 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <libselftest.h>
-#include <pk-debug.h>
+#include "egg-test.h"
+#include "egg-debug.h"
 #include "gpk-common.h"
+#include "gpk-enum.h"
+#include "gpk-task.h"
+
+void egg_markdown_test (EggTest *test);
+void egg_string_test (EggTest *test);
+void gpk_dbus_test (EggTest *test);
+void gpk_language_test (EggTest *test);
+void gpk_modal_dialog_test (EggTest *test);
+void gpk_client_test (EggTest *test);
+void gpk_error_test (EggTest *test);
 
 int
 main (int argc, char **argv)
 {
-	LibSelfTest test;
+	EggTest *test;
 
-	if (! g_thread_supported ()) {
+	if (! g_thread_supported ())
 		g_thread_init (NULL);
-	}
 	g_type_init ();
-	libst_init (&test);
-	pk_debug_init (TRUE);
+	gtk_init (&argc, &argv);
+	test = egg_test_init ();
+	egg_debug_init (&argc, &argv);
 
 	/* tests go here */
-	gpk_common_self_test (&test);
+	egg_markdown_test (test);
+	egg_string_test (test);
+	gpk_enum_test (test);
+	gpk_common_test (test);
+//	gpk_dbus_test (test);
+	gpk_language_test (test);
+	gpk_error_test (test);
+//	gpk_client_test (test);
+	gpk_modal_dialog_test (test);
+	gpk_task_test (test);
 
-	return (libst_finish (&test));
+	return egg_test_finish (test);
 }
 
